/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include "Heavy_WSTD_FLDR.hpp"

#include <new>

#define Context(_c) static_cast<Heavy_WSTD_FLDR *>(_c)


/*
 * C Functions
 */

extern "C" {
  HV_EXPORT HeavyContextInterface *hv_WSTD_FLDR_new(double sampleRate) {
    // allocate aligned memory
    void *ptr = hv_malloc(sizeof(Heavy_WSTD_FLDR));
    // ensure non-null
    if (!ptr) return nullptr;
    // call constructor
    new(ptr) Heavy_WSTD_FLDR(sampleRate);
    return Context(ptr);
  }

  HV_EXPORT HeavyContextInterface *hv_WSTD_FLDR_new_with_options(double sampleRate,
      int poolKb, int inQueueKb, int outQueueKb) {
    // allocate aligned memory
    void *ptr = hv_malloc(sizeof(Heavy_WSTD_FLDR));
    // ensure non-null
    if (!ptr) return nullptr;
    // call constructor
    new(ptr) Heavy_WSTD_FLDR(sampleRate, poolKb, inQueueKb, outQueueKb);
    return Context(ptr);
  }

  HV_EXPORT void hv_WSTD_FLDR_free(HeavyContextInterface *instance) {
    // call destructor
    Context(instance)->~Heavy_WSTD_FLDR();
    // free memory
    hv_free(instance);
  }
} // extern "C"







/*
 * Class Functions
 */

Heavy_WSTD_FLDR::Heavy_WSTD_FLDR(double sampleRate, int poolKb, int inQueueKb, int outQueueKb)
    : HeavyContext(sampleRate, poolKb, inQueueKb, outQueueKb) {
  numBytes += sLine_init(&sLine_JIPLVf3k);
  numBytes += sEnv_init(&sEnv_L3wRPgos, 512, 256);
  numBytes += sEnv_init(&sEnv_xP4SO5ae, 512, 256);
  numBytes += sEnv_init(&sEnv_xt8tqMaf, 512, 256);
  numBytes += sEnv_init(&sEnv_99HwurPb, 512, 256);
  numBytes += sLine_init(&sLine_qUybwYRN);
  numBytes += sLine_init(&sLine_QWvEXC9O);
  numBytes += cVar_init_f(&cVar_8wX0xG5o, 0.0f);
  numBytes += cVar_init_f(&cVar_9l492nng, 0.0f);
  numBytes += sVarf_init(&sVarf_8AhNzfxK, 0.0f, 0.0f, false);
  numBytes += cVar_init_f(&cVar_w2MA37St, 0.0f);
  numBytes += cIf_init(&cIf_UzLrs9Fm, false);
  numBytes += cIf_init(&cIf_Ii4hvtAc, false);
  numBytes += cBinop_init(&cBinop_f2i3yDj0, 0.0f); // __sub
  numBytes += sVarf_init(&sVarf_ZGMBzzdu, 0.0f, 0.0f, false);
  numBytes += cVar_init_f(&cVar_g93WI8W1, 0.0f);
  numBytes += cIf_init(&cIf_Y8hokiiq, false);
  numBytes += cIf_init(&cIf_DzzHUsR0, false);
  numBytes += cBinop_init(&cBinop_UQrEz5Q9, 0.0f); // __sub
  numBytes += cVar_init_f(&cVar_1MulBgdQ, 1.0f);
  numBytes += sVarf_init(&sVarf_Eo6Eacvp, 0.0f, 0.0f, false);
  numBytes += sVarf_init(&sVarf_bRmq9q3H, 0.0f, 0.0f, false);
  numBytes += sVarf_init(&sVarf_Vljl9adU, 0.0f, 0.0f, false);
  numBytes += sVarf_init(&sVarf_lufYrqle, 0.0f, 0.0f, false);
  
  // schedule a message to trigger all loadbangs via the __hv_init receiver
  scheduleMessageForReceiver(0xCE5CC65B, msg_initWithBang(HV_MESSAGE_ON_STACK(1), 0));
}

Heavy_WSTD_FLDR::~Heavy_WSTD_FLDR() {
  sEnv_free(&sEnv_L3wRPgos);
  sEnv_free(&sEnv_xP4SO5ae);
  sEnv_free(&sEnv_xt8tqMaf);
  sEnv_free(&sEnv_99HwurPb);
}

HvTable *Heavy_WSTD_FLDR::getTableForHash(hv_uint32_t tableHash) {
  return nullptr;
}

void Heavy_WSTD_FLDR::scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) {
  switch (receiverHash) {
    case 0xE42C892E: { // Fldr
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_SEDJFhG7_sendMessage);
      break;
    }
    case 0xCB23DC60: { // Lmtr
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_Kcu5nEhO_sendMessage);
      break;
    }
    case 0x97D46AC6: { // Mix
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_h1jJV6n2_sendMessage);
      break;
    }
    case 0xCF8296B9: { // 1001-mix
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_hZpYD8vl_sendMessage);
      break;
    }
    case 0xCE5CC65B: { // __hv_init
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_f518JlW7_sendMessage);
      break;
    }
    default: return;
  }
}

int Heavy_WSTD_FLDR::getParameterInfo(int index, HvParameterInfo *info) {
  if (info != nullptr) {
    switch (index) {
      case 0: {
        info->name = "Fldr";
        info->hash = 0xE42C892E;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 1.0f;
        info->maxVal = 13.37f;
        info->defaultVal = 1.0f;
        break;
      }
      case 1: {
        info->name = "Lmtr";
        info->hash = 0xCB23DC60;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 0.0f;
        info->maxVal = 1.0f;
        info->defaultVal = 1.0f;
        break;
      }
      case 2: {
        info->name = "Mix";
        info->hash = 0x97D46AC6;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 0.0f;
        info->maxVal = 100.0f;
        info->defaultVal = 50.0f;
        break;
      }
      default: {
        info->name = "invalid parameter index";
        info->hash = 0;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 0.0f;
        info->maxVal = 0.0f;
        info->defaultVal = 0.0f;
        break;
      }
    }
  }
  return 3;
}



/*
 * Send Function Implementations
 */


void Heavy_WSTD_FLDR::cBinop_0hcLzqn6_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MAX, 0.0f, 0, m, &cBinop_j4CGos3g_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_j4CGos3g_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MULTIPLY, 2.0f, 0, m, &cBinop_ynrmLf9P_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_ynrmLf9P_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 1.0f, 0, m, &cBinop_V9wwlYtj_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_V9wwlYtj_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_QjAxc4U1_sendMessage);
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_KGWdWstM_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_QjAxc4U1_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_ADD, 1.0f, 0, m, &cBinop_10LEhanw_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_KGWdWstM_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_nYvPq1UO_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cBinop_10LEhanw_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MULTIPLY, 0.5f, 0, m, &cBinop_qF4V0alM_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_nYvPq1UO_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setFloat(m, 0, 1.0f);
  msg_setElementToFrom(m, 1, n, 0);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 0.0f, 0, m, &cBinop_1orL7Zak_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_1orL7Zak_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MULTIPLY, 0.5f, 0, m, &cBinop_dPFTjtCP_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_qF4V0alM_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_VFhNQVbh_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cBinop_dPFTjtCP_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_LtgMJoZ1_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cVar_8wX0xG5o_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cSend_akGbSvzK_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cCast_qvm3P7gY_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_4XQ8Bms0_sendMessage(_c, 0, m);
  cMsg_yhshk9ye_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cVar_9l492nng_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_3CLSu1Z0_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cVar_w2MA37St_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_f2i3yDj0, HV_BINOP_SUBTRACT, 0, m, &cBinop_f2i3yDj0_sendMessage);
}

void Heavy_WSTD_FLDR::cIf_UzLrs9Fm_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_8AhNzfxK, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_FLDR::cIf_Ii4hvtAc_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_8AhNzfxK, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_FLDR::sEnv_L3wRPgos_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_w2MA37St, 0, m, &cVar_w2MA37St_sendMessage);
}

void Heavy_WSTD_FLDR::sEnv_xP4SO5ae_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_2dvSlXGk_sendMessage);
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_5orTDQjx_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_wcvPZpAG_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_f2i3yDj0_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_uFJPM4iv_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cMsg_uFJPM4iv_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  msg_setElementToFrom(m, 1, n, 0);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 0.0f, 0, m, &cBinop_kcoAml07_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_kcoAml07_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MIN, 100.0f, 0, m, &cBinop_4b5C0yVg_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_xH30WpqN_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_uaG19mtQ_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_l4il8fOA_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_uaG19mtQ_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_UzLrs9Fm, 1, m, &cIf_UzLrs9Fm_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_l4il8fOA_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_HnUqEiNm_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cBinop_jQkAMpjH_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_Ii4hvtAc, 1, m, &cIf_Ii4hvtAc_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_HnUqEiNm_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  cIf_onMessage(_c, &Context(_c)->cIf_UzLrs9Fm, 0, m, &cIf_UzLrs9Fm_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_4b5C0yVg_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_Ii4hvtAc, 0, m, &cIf_Ii4hvtAc_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_2dvSlXGk_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
}

void Heavy_WSTD_FLDR::cCast_wcvPZpAG_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_w2MA37St, 0, m, &cVar_w2MA37St_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_5orTDQjx_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_f2i3yDj0, HV_BINOP_SUBTRACT, 1, m, &cBinop_f2i3yDj0_sendMessage);
}

void Heavy_WSTD_FLDR::cVar_g93WI8W1_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_UQrEz5Q9, HV_BINOP_SUBTRACT, 0, m, &cBinop_UQrEz5Q9_sendMessage);
}

void Heavy_WSTD_FLDR::cIf_Y8hokiiq_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_ZGMBzzdu, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_FLDR::cIf_DzzHUsR0_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_ZGMBzzdu, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_FLDR::sEnv_xt8tqMaf_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_g93WI8W1, 0, m, &cVar_g93WI8W1_sendMessage);
}

void Heavy_WSTD_FLDR::sEnv_99HwurPb_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_hnTuNouK_sendMessage);
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_iy2pNCAx_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_G9uQug9z_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_UQrEz5Q9_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_QurGGMBK_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cMsg_QurGGMBK_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  msg_setElementToFrom(m, 1, n, 0);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 0.0f, 0, m, &cBinop_drSQQuNn_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_drSQQuNn_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MIN, 100.0f, 0, m, &cBinop_nFq3tZLI_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_i3ON8BFL_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_imb8YLel_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_57N4ughr_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_imb8YLel_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_Y8hokiiq, 1, m, &cIf_Y8hokiiq_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_57N4ughr_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_o5ypMPiQ_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cBinop_fRrvRceR_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_DzzHUsR0, 1, m, &cIf_DzzHUsR0_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_o5ypMPiQ_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  cIf_onMessage(_c, &Context(_c)->cIf_Y8hokiiq, 0, m, &cIf_Y8hokiiq_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_nFq3tZLI_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_DzzHUsR0, 0, m, &cIf_DzzHUsR0_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_hnTuNouK_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
}

void Heavy_WSTD_FLDR::cCast_iy2pNCAx_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_UQrEz5Q9, HV_BINOP_SUBTRACT, 1, m, &cBinop_UQrEz5Q9_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_G9uQug9z_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_g93WI8W1, 0, m, &cVar_g93WI8W1_sendMessage);
}

void Heavy_WSTD_FLDR::cVar_1MulBgdQ_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 0.0f, 0, m, &cBinop_Lp0rHmmF_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 0.0f, 0, m, &cBinop_xH30WpqN_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 1.0f, 0, m, &cBinop_jQkAMpjH_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 0.0f, 0, m, &cBinop_i3ON8BFL_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 1.0f, 0, m, &cBinop_fRrvRceR_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_Lp0rHmmF_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_1MulBgdQ, 1, m, &cVar_1MulBgdQ_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_VFhNQVbh_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setElementToFrom(m, 0, n, 0);
  msg_setFloat(m, 1, 100.0f);
  sLine_onMessage(_c, &Context(_c)->sLine_QWvEXC9O, 0, m, NULL);
}

void Heavy_WSTD_FLDR::cMsg_LtgMJoZ1_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setElementToFrom(m, 0, n, 0);
  msg_setFloat(m, 1, 100.0f);
  sLine_onMessage(_c, &Context(_c)->sLine_qUybwYRN, 0, m, NULL);
}

void Heavy_WSTD_FLDR::cSend_akGbSvzK_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cReceive_hZpYD8vl_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cBinop_sRLrIsxw_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_8wX0xG5o, 0, m, &cVar_8wX0xG5o_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_yhshk9ye_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 50.0f);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_DIVIDE, 100.0f, 0, m, &cBinop_sRLrIsxw_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_3CLSu1Z0_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setElementToFrom(m, 0, n, 0);
  msg_setFloat(m, 1, 20.0f);
  sLine_onMessage(_c, &Context(_c)->sLine_JIPLVf3k, 0, m, NULL);
}

void Heavy_WSTD_FLDR::cMsg_4XQ8Bms0_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 1.0f);
  cVar_onMessage(_c, &Context(_c)->cVar_9l492nng, 0, m, &cVar_9l492nng_sendMessage);
  cVar_onMessage(_c, &Context(_c)->cVar_1MulBgdQ, 0, m, &cVar_1MulBgdQ_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_hZpYD8vl_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MIN, 1.0f, 0, m, &cBinop_0hcLzqn6_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_h1jJV6n2_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_DIVIDE, 100.0f, 0, m, &cBinop_sRLrIsxw_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_f518JlW7_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_qvm3P7gY_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_Kcu5nEhO_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_1MulBgdQ, 0, m, &cVar_1MulBgdQ_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_SEDJFhG7_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_9l492nng, 0, m, &cVar_9l492nng_sendMessage);
}




/*
 * Context Process Implementation
 */

int Heavy_WSTD_FLDR::process(float **inputBuffers, float **outputBuffers, int n) {
  while (hLp_hasData(&inQueue)) {
    hv_uint32_t numBytes = 0;
    ReceiverMessagePair *p = reinterpret_cast<ReceiverMessagePair *>(hLp_getReadBuffer(&inQueue, &numBytes));
    hv_assert(numBytes >= sizeof(ReceiverMessagePair));
    scheduleMessageForReceiver(p->receiverHash, &p->msg);
    hLp_consume(&inQueue);
  }

  sendBangToReceiver(0xDD21C0EB); // send to __hv_bang~ on next cycle
  const int n4 = n & ~HV_N_SIMD_MASK; // ensure that the block size is a multiple of HV_N_SIMD

  // temporary signal vars
  hv_bufferf_t Bf0, Bf1, Bf2, Bf3;

  // input and output vars
  hv_bufferf_t O0, O1;
  hv_bufferf_t I0, I1;

  // declare and init the zero buffer
  hv_bufferf_t ZERO; __hv_zero_f(VOf(ZERO));

  hv_uint32_t nextBlock = blockStartTimestamp;
  for (int n = 0; n < n4; n += HV_N_SIMD) {

    // process all of the messages for this block
    nextBlock += HV_N_SIMD;
    while (mq_hasMessageBefore(&mq, nextBlock)) {
      MessageNode *const node = mq_peek(&mq);
      node->sendMessage(this, node->let, node->m);
      mq_pop(&mq);
    }

    // load input buffers
    __hv_load_f(inputBuffers[0]+n, VOf(I0));
    __hv_load_f(inputBuffers[1]+n, VOf(I1));

    // zero output buffers
    __hv_zero_f(VOf(O0));
    __hv_zero_f(VOf(O1));

    // process all signal functions
    __hv_varwrite_f(&sVarf_Eo6Eacvp, VIf(I0));
    __hv_line_f(&sLine_JIPLVf3k, VOf(Bf0));
    __hv_var_k_f(VOf(Bf1), 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f);
    __hv_fma_f(VIf(I0), VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf2), 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f);
    __hv_div_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_floor_f(VIf(Bf2), VOf(Bf1));
    __hv_sub_f(VIf(Bf2), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf2), 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f);
    __hv_sub_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_abs_f(VIf(Bf2), VOf(Bf2));
    __hv_var_k_f(VOf(Bf1), 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f);
    __hv_var_k_f(VOf(Bf3), 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f);
    __hv_fms_f(VIf(Bf2), VIf(Bf1), VIf(Bf3), VOf(Bf3));
    __hv_neg_f(VIf(Bf3), VOf(Bf3));
    sEnv_process(this, &sEnv_L3wRPgos, VIf(Bf3), &sEnv_L3wRPgos_sendMessage);
    sEnv_process(this, &sEnv_xP4SO5ae, VIf(I0), &sEnv_xP4SO5ae_sendMessage);
    __hv_varread_f(&sVarf_8AhNzfxK, VOf(Bf1));
    __hv_var_k_f(VOf(Bf2), 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f);
    __hv_sub_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_var_k_f(VOf(Bf1), 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f);
    __hv_mul_f(VIf(Bf2), VIf(Bf1), VOf(Bf1));
    __hv_exp_f(VIf(Bf1), VOf(Bf1));
    __hv_mul_f(VIf(Bf3), VIf(Bf1), VOf(Bf1));
    __hv_varwrite_f(&sVarf_bRmq9q3H, VIf(Bf1));
    __hv_varwrite_f(&sVarf_Vljl9adU, VIf(I1));
    __hv_var_k_f(VOf(Bf1), 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f);
    __hv_fma_f(VIf(I1), VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf0), 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f);
    __hv_div_f(VIf(Bf1), VIf(Bf0), VOf(Bf0));
    __hv_floor_f(VIf(Bf0), VOf(Bf1));
    __hv_sub_f(VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf0), 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f);
    __hv_sub_f(VIf(Bf1), VIf(Bf0), VOf(Bf0));
    __hv_abs_f(VIf(Bf0), VOf(Bf0));
    __hv_var_k_f(VOf(Bf1), 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f);
    __hv_var_k_f(VOf(Bf3), 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f);
    __hv_fms_f(VIf(Bf0), VIf(Bf1), VIf(Bf3), VOf(Bf3));
    __hv_neg_f(VIf(Bf3), VOf(Bf3));
    sEnv_process(this, &sEnv_xt8tqMaf, VIf(Bf3), &sEnv_xt8tqMaf_sendMessage);
    sEnv_process(this, &sEnv_99HwurPb, VIf(I1), &sEnv_99HwurPb_sendMessage);
    __hv_varread_f(&sVarf_ZGMBzzdu, VOf(Bf1));
    __hv_var_k_f(VOf(Bf0), 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f);
    __hv_sub_f(VIf(Bf1), VIf(Bf0), VOf(Bf0));
    __hv_var_k_f(VOf(Bf1), 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f);
    __hv_mul_f(VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_exp_f(VIf(Bf1), VOf(Bf1));
    __hv_mul_f(VIf(Bf3), VIf(Bf1), VOf(Bf1));
    __hv_varwrite_f(&sVarf_lufYrqle, VIf(Bf1));
    __hv_varread_f(&sVarf_Eo6Eacvp, VOf(Bf1));
    __hv_line_f(&sLine_qUybwYRN, VOf(Bf3));
    __hv_varread_f(&sVarf_bRmq9q3H, VOf(Bf0));
    __hv_line_f(&sLine_QWvEXC9O, VOf(Bf2));
    __hv_mul_f(VIf(Bf0), VIf(Bf2), VOf(Bf0));
    __hv_fma_f(VIf(Bf1), VIf(Bf3), VIf(Bf0), VOf(Bf0));
    __hv_add_f(VIf(Bf0), VIf(O0), VOf(O0));
    __hv_varread_f(&sVarf_Vljl9adU, VOf(Bf0));
    __hv_varread_f(&sVarf_lufYrqle, VOf(Bf1));
    __hv_mul_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_fma_f(VIf(Bf0), VIf(Bf3), VIf(Bf2), VOf(Bf2));
    __hv_add_f(VIf(Bf2), VIf(O1), VOf(O1));

    // save output vars to output buffer
    __hv_store_f(outputBuffers[0]+n, VIf(O0));
    __hv_store_f(outputBuffers[1]+n, VIf(O1));
  }

  blockStartTimestamp = nextBlock;

  return n4; // return the number of frames processed

}

int Heavy_WSTD_FLDR::processInline(float *inputBuffers, float *outputBuffers, int n4) {
  hv_assert(!(n4 & HV_N_SIMD_MASK)); // ensure that n4 is a multiple of HV_N_SIMD

  // define the heavy input buffer for 2 channel(s)
  float **const bIn = reinterpret_cast<float **>(hv_alloca(2*sizeof(float *)));
  bIn[0] = inputBuffers+(0*n4);
  bIn[1] = inputBuffers+(1*n4);

  // define the heavy output buffer for 2 channel(s)
  float **const bOut = reinterpret_cast<float **>(hv_alloca(2*sizeof(float *)));
  bOut[0] = outputBuffers+(0*n4);
  bOut[1] = outputBuffers+(1*n4);

  int n = process(bIn, bOut, n4);
  return n;
}

int Heavy_WSTD_FLDR::processInlineInterleaved(float *inputBuffers, float *outputBuffers, int n4) {
  hv_assert(n4 & ~HV_N_SIMD_MASK); // ensure that n4 is a multiple of HV_N_SIMD

  // define the heavy input buffer for 2 channel(s), uninterleave
  float *const bIn = reinterpret_cast<float *>(hv_alloca(2*n4*sizeof(float)));
  #if HV_SIMD_SSE || HV_SIMD_AVX
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    __m128 a = _mm_load_ps(inputBuffers+i);                // LRLR
    __m128 b = _mm_load_ps(inputBuffers+4+i);              // LRLR
    __m128 x = _mm_shuffle_ps(a, b, _MM_SHUFFLE(2,0,2,0)); // LLLL
    __m128 y = _mm_shuffle_ps(a, b, _MM_SHUFFLE(3,1,3,1)); // RRRR
    _mm_store_ps(bIn+j, x);
    _mm_store_ps(bIn+n4+j, y);
  }
  #elif HV_SIMD_NEON
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    float32x4x2_t a = vld2q_f32(inputBuffers+i); // load and uninterleave
    vst1q_f32(bIn+j, a.val[0]);
    vst1q_f32(bIn+n4+j, a.val[1]);
  }
  #else // HV_SIMD_NONE
  for (int j = 0; j < n4; ++j) {
    bIn[0*n4+j] = inputBuffers[0+2*j];
    bIn[1*n4+j] = inputBuffers[1+2*j];
  }
  #endif

  // define the heavy output buffer for 2 channel(s)
  float *const bOut = reinterpret_cast<float *>(hv_alloca(2*n4*sizeof(float)));

  int n = processInline(bIn, bOut, n4);

  // interleave the heavy output into the output buffer
  #if HV_SIMD_AVX
  for (int i = 0, j = 0; j < n4; j += 8, i += 16) {
    __m256 x = _mm256_load_ps(bOut+j);    // LLLLLLLL
    __m256 y = _mm256_load_ps(bOut+n4+j); // RRRRRRRR
    __m256 a = _mm256_unpacklo_ps(x, y);  // LRLRLRLR
    __m256 b = _mm256_unpackhi_ps(x, y);  // LRLRLRLR
    _mm256_store_ps(outputBuffers+i, a);
    _mm256_store_ps(outputBuffers+8+i, b);
  }
  #elif HV_SIMD_SSE
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    __m128 x = _mm_load_ps(bOut+j);    // LLLL
    __m128 y = _mm_load_ps(bOut+n4+j); // RRRR
    __m128 a = _mm_unpacklo_ps(x, y);  // LRLR
    __m128 b = _mm_unpackhi_ps(x, y);  // LRLR
    _mm_store_ps(outputBuffers+i, a);
    _mm_store_ps(outputBuffers+4+i, b);
  }
  #elif HV_SIMD_NEON
  // https://community.arm.com/groups/processors/blog/2012/03/13/coding-for-neon--part-5-rearranging-vectors
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    float32x4_t x = vld1q_f32(bOut+j);
    float32x4_t y = vld1q_f32(bOut+n4+j);
    float32x4x2_t z = {x, y};
    vst2q_f32(outputBuffers+i, z); // interleave and store
  }
  #else // HV_SIMD_NONE
  for (int i = 0; i < 2; ++i) {
    for (int j = 0; j < n4; ++j) {
      outputBuffers[i+2*j] = bOut[i*n4+j];
    }
  }
  #endif

  return n;
}
