/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_FLDR_HPP_
#define _HEAVY_CONTEXT_WSTD_FLDR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlVar.h"
#include "HvControlIf.h"
#include "HvSignalLine.h"
#include "HvMath.h"
#include "HvSignalVar.h"
#include "HvSignalEnvelope.h"
#include "HvControlCast.h"
#include "HvControlBinop.h"

class Heavy_WSTD_FLDR : public HeavyContext {

 public:
  Heavy_WSTD_FLDR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_FLDR();

  const char *getName() override { return "WSTD_FLDR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        FLDR = 0xE42C892E, // Fldr
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_0hcLzqn6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_j4CGos3g_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ynrmLf9P_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_V9wwlYtj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_QjAxc4U1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_KGWdWstM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_10LEhanw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_nYvPq1UO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_1orL7Zak_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_qF4V0alM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_dPFTjtCP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_8wX0xG5o_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_qvm3P7gY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_9l492nng_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_w2MA37St_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_UzLrs9Fm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_Ii4hvtAc_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_L3wRPgos_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_xP4SO5ae_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_f2i3yDj0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_uFJPM4iv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_kcoAml07_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_xH30WpqN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_uaG19mtQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_l4il8fOA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_jQkAMpjH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_HnUqEiNm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_4b5C0yVg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_2dvSlXGk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_wcvPZpAG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_5orTDQjx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_g93WI8W1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_Y8hokiiq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_DzzHUsR0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_xt8tqMaf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_99HwurPb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_UQrEz5Q9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_QurGGMBK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_drSQQuNn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_i3ON8BFL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_imb8YLel_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_57N4ughr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_fRrvRceR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_o5ypMPiQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_nFq3tZLI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_hnTuNouK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_iy2pNCAx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_G9uQug9z_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_1MulBgdQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Lp0rHmmF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_VFhNQVbh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_LtgMJoZ1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_akGbSvzK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_sRLrIsxw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_yhshk9ye_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_3CLSu1Z0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_4XQ8Bms0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_hZpYD8vl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_h1jJV6n2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_f518JlW7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_Kcu5nEhO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_SEDJFhG7_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_JIPLVf3k;
  SignalEnvelope sEnv_L3wRPgos;
  SignalEnvelope sEnv_xP4SO5ae;
  SignalEnvelope sEnv_xt8tqMaf;
  SignalEnvelope sEnv_99HwurPb;
  SignalLine sLine_qUybwYRN;
  SignalLine sLine_QWvEXC9O;
  ControlBinop cBinop_0hcLzqn6;
  ControlBinop cBinop_j4CGos3g;
  ControlBinop cBinop_ynrmLf9P;
  ControlBinop cBinop_V9wwlYtj;
  ControlBinop cBinop_10LEhanw;
  ControlBinop cBinop_1orL7Zak;
  ControlBinop cBinop_qF4V0alM;
  ControlBinop cBinop_dPFTjtCP;
  ControlVar cVar_8wX0xG5o;
  ControlVar cVar_9l492nng;
  SignalVarf sVarf_8AhNzfxK;
  ControlVar cVar_w2MA37St;
  ControlIf cIf_UzLrs9Fm;
  ControlIf cIf_Ii4hvtAc;
  ControlBinop cBinop_f2i3yDj0;
  ControlBinop cBinop_kcoAml07;
  ControlBinop cBinop_xH30WpqN;
  ControlBinop cBinop_jQkAMpjH;
  ControlBinop cBinop_4b5C0yVg;
  SignalVarf sVarf_ZGMBzzdu;
  ControlVar cVar_g93WI8W1;
  ControlIf cIf_Y8hokiiq;
  ControlIf cIf_DzzHUsR0;
  ControlBinop cBinop_UQrEz5Q9;
  ControlBinop cBinop_drSQQuNn;
  ControlBinop cBinop_i3ON8BFL;
  ControlBinop cBinop_fRrvRceR;
  ControlBinop cBinop_nFq3tZLI;
  ControlVar cVar_1MulBgdQ;
  ControlBinop cBinop_Lp0rHmmF;
  SignalVarf sVarf_Eo6Eacvp;
  SignalVarf sVarf_bRmq9q3H;
  SignalVarf sVarf_Vljl9adU;
  SignalVarf sVarf_lufYrqle;
  ControlBinop cBinop_sRLrIsxw;
};

#endif // _HEAVY_CONTEXT_WSTD_FLDR_HPP_
